package BPpackage;

import android.content.Context;
import android.content.res.AssetManager;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Reader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.lang.Runnable;
import java.lang.Thread;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
   PRAFileHandler 2.10
added updateInAllByToken0(String line, String sep)
 ***   (public methods)
   copyAssets
   copyDirectory
   copyExtFile
   deleteRecursive
   getAssetAsArray
   getAssetAsList
   getExtFileAsArray
   getExtFileAsList
   getFileAsArray
   getFileAsList
   getLineByToken0
   inputStreamToString
   listAssets
   move
   remove
   removeExt
   removeLineByToken0
   touchFile
   updateInAllByToken0
   writeArrayToExtFile
   writeArrayToFile
   writeListToFile
   writeToTopOfFile
   writeUniqLineByToken0
   writeUniqLineByTokens
 ***   (private methods)
   mCopyFile (used by copyAssets)
 **/

public class PRAFileHandler {

	public Context mContext;

	public PRAFileHandler(Context context) {
		mContext = context;
	}



	/**
	 * copies asset files with extension "ext" to target_dir
	 * does not check for external write permissions
	 **/
	public void copyAssets(String target_dir, String ext) {
		AssetManager assetManager = mContext.getAssets();
		String[] files = null;
		try {
			files = assetManager.list("");
		} catch (IOException e) {
			// empty
		}
		for (String filename : files) {
			InputStream in = null;
			OutputStream out = null;
			if (filename.endsWith(ext)) {
				try {
					in = assetManager.open(filename);
					out = new FileOutputStream(target_dir + "/" + filename);
					mCopyFile(in, out);
					in.close();
					in = null;
					out.flush();
					out.close();
					out = null;
				} catch (IOException e) {
					// empty
				}
			}
		}
	}



	/**
	 *   To copy a directory: src, dst, false
	 *   To move a directory: src, dst, true; then YOU must call deleteRecursive(src) 
	 **/
	public void copyDirectory(String orig, String dst, boolean deleteOrig) {
		String[] tokens	= orig.split("/");
		String dname = tokens[tokens.length - 1];
		File dest = new File(dst + "/" + dname);
		dest.mkdir();

		File src = new File(orig);
		File[] list = src.listFiles();
		for (File f : list){
			if (f.isFile()){
				copyExtFile(f.toString(), dest.toString(), deleteOrig);
			} else {
				String fs = f.toString();
				if (fs.length() > 0 && fs.charAt(fs.length()-1)=='/') {
					fs = fs.substring(0, fs.length()-1);
					f = new File(fs);
				}
				copyDirectory(f.toString(), dest.toString(), deleteOrig);
			}
		}
		
	}



	/**
	 * copies orig to dest, use deleteOrig true for mv
	 **/
	public void copyExtFile(String orig, String dst, boolean deleteOrig) {
		final String src = orig;
		final String dest = dst;
		final boolean move = deleteOrig;

		new Thread(new Runnable() {
				public void run() {

					String[] tokens	= src.split("/");
					String inputFile = tokens[tokens.length - 1];
					if (src.matches(dest +"/" + inputFile)){
						return;
					}
					
					InputStream in = null;
					OutputStream out = null;
					
					try {
						in = new FileInputStream (src);        
						out = new FileOutputStream (dest +"/" + inputFile);
						byte[] buffer = new byte[1024];
						int read;
						while ((read = in.read(buffer)) != -1) {
							out.write(buffer, 0, read);
						}
						
						in.close();
						in = null;
						out.flush();
						out.close();
						out = null;
						
						if (move){
							new File(src).delete();  
						}
					} 
					catch (FileNotFoundException fnfe1) {
						;
					}
					catch (Exception e) {
						;
					}
				}
			}).start();
		
	}//END copyExtFile



	/**
	 * Recursively deletes empty dir "path"
	 **/
    public void deleteRecursive(File path){
        if (path.isDirectory()){
            for (File f : path.listFiles()){
                deleteRecursive(f);
            }
        }
        path.delete();
    }	



	/**
	 * Reads asset with path "fname" into String[]
	 * return String[] which is empty on error
	 */
	public String[] getAssetAsArray(String fname) {
		List<String> ifile = getAssetAsList(fname);
		String[] file = new String[ifile.size()];
		ifile.toArray(file);
		return file;
	}



	/**
	 * Reads asset with path "fname" into List<String>
	 * return List which is empty on error
	 */
	public List<String> getAssetAsList(String fname) {
		List<String> list = new ArrayList<String>();
		AssetManager am = mContext.getAssets();
		InputStream is;
		String line;
		try {
			is = am.open(fname);
			BufferedReader br = new BufferedReader(new InputStreamReader(
					is));
			while ((line = br.readLine()) != null) {
				list.add(line);
			}
		} catch (IOException e) {
			// empty
		}
		return list;
	}



	/**
	 * return String[] of all lines in external file dir/name
	 * empty on error
	 */
	public String[] getExtFileAsArray(String dir, String fname) {
		List<String> ifile = getExtFileAsList(dir, fname);
		String[] file = new String[ifile.size()];
		ifile.toArray(file);
		return file;
	}



	/**
	 * return List of String of all lines in external file dir/name
	 * empty on error
	 */
	public List<String> getExtFileAsList(String dir, String fname) {
		File file = new File(dir, fname);
		List<String> ifile = new ArrayList<String>();

		try {
			BufferedReader br = new BufferedReader(new FileReader(file));
			String line;

			while ((line = br.readLine()) != null) {
				ifile.add(line);
			}
		} catch (IOException e) {
			// empty
		}
		return ifile;
	}



	/**
	 * return String[] of all lines in internal file fname
	 * empty on error
	 */
	public String[] getFileAsArray(String fname) {
		List<String> ifile = getFileAsList(fname);
		String[] file = new String[ifile.size()];
		ifile.toArray(file);
		return file;
	}



	/**
	 * Reads internal file fname into List<String>
	 * return null on error, else List<String> of lines in file
	 */
	public List<String> getFileAsList(String fname) {
		List<String> ifile = new ArrayList<String>();
		String line;
		BufferedReader r = null;
		try {
			r = new BufferedReader(new InputStreamReader(
					mContext.openFileInput(fname)));
			while ((line = r.readLine()) != null) {
				ifile.add(new String(line));
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				if (r != null) {
					r.close();
				}
			} catch (IOException io) {
				io.printStackTrace();
			}
		}

		try {
			if (r != null) {
				r.close();
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return ifile;
	}


	/**
	 * Returns line in internal file fname by unique token[0] 
	 * Sep is seperator character in line
	 * Returns "" if line not found
	 */
	public String getLineByToken0(String fname, String token, String sep) {
		String[] file = getFileAsArray(fname);

		for (String string : file) {
			String tmp2[] = string.split(sep);
			String old0 = tmp2[0];
			if (old0.matches(token)) {
				return string;
			}
		}

		return "";
	}



	/**
	 * Reads external file filename into a single String
	 * Not sure what you get on error yet. Null?
	 */
	public String inputStreamToString(String filename) {
		File file = new File(filename);
		InputStream iFile = null;
		try {
			iFile = new FileInputStream(file);
		} catch (FileNotFoundException e1) {
			// empty
		}
		Writer writer = new StringWriter();
		if (iFile != null) {
			char[] buffer = new char[1024];
			try {
				Reader reader = new BufferedReader(new InputStreamReader(
						iFile, "UTF-8"));
				int n;
				while ((n = reader.read(buffer)) != -1) {
					writer.write(buffer, 0, n);
				}
			} catch (UnsupportedEncodingException e) {
				return e.toString();
			} catch (IOException e) {
				return e.toString();
			} finally {
				try {
					iFile.close();
				} catch (IOException e) {
					return e.toString();
				}
			}
		}
		String result = writer.toString();
		return result;
	}



	/**
	 * returns list of asset files with extension "ext" as String[]
	 **/
	public String[] listAssets(String ext) {
		AssetManager assetManager = mContext.getAssets();
		String[] list = null;
		try {
			list = assetManager.list("");
		} catch (IOException e) {
			; //empty
		}
		List<String> alist = new ArrayList<String>();
		for (String f : list){
			if (f.endsWith(ext)){
				alist.add(f);
			}
		}
		String[] files = new String[alist.size()];
		alist.toArray(files);

		return files;
	}



	/**
	 * used by copyAssets 
	 * copies all files with extension ext to target_dir 
	 * param ext must include dot, ie ".txt"
	 */
	private void mCopyFile(InputStream in, OutputStream out) throws IOException {
		byte[] buffer = new byte[1024];
		int read;
		while ((read = in.read(buffer)) != -1) {
			out.write(buffer, 0, read);
		}
	}



	public void move(String oldname, String newname){
		File oldf = new File(oldname);
		File newf = new File(newname);
		oldf.renameTo(newf);
	}


	/**
	 * deletes internal fname
	 */
	public void remove(String fname) {
		mContext.deleteFile(fname);
	}



	/**
	 * deletes external fname
	 */
	public boolean removeExt(String fname) {
		File tmp = new File(fname);
		return tmp.delete();
	}



	/**
	 * Removes line in internal file fname by unique token[0]
	 * Removes all from token except 0-9A-Za-z
	 * Separator of lines in fname is sep
	 * return -1 on error else 0
	 */
	public void removeLineByToken0(String fname, String token0, String sep) {
		String[] file = getFileAsArray(fname);
		FileOutputStream writer = null;

		try {
			writer = mContext.openFileOutput(fname, Context.MODE_PRIVATE);
		} catch (FileNotFoundException e) {
			// empty
		}

		for (String string : file) {
			if (string == null) {
				break;
			}
			String token = string.substring(0, string.indexOf(sep));
			token0 = token0.replaceAll("[^0-9A-Za-z\\s]", "");
			token = token.replaceAll("[^0-9A-Za-z\\s]", "");
			if (!token0.matches(token)) {
				try {
					writer.write(string.getBytes());
					writer.write(System.getProperty("line.separator")
							.getBytes());
					writer.flush();
				} catch (IOException e) {
					// empty
				}
			}
		}
		try {
			writer.close();
		} catch (IOException e) {
			// empty
		}
	}



	/**
	 * Removes line in internal file fname by unique token[0]
	 * Separator of lines in fname is sep
	 * return -1 on error else 0
	 */
	public void removeLineByFullToken0(String fname, String token0, String sep) {
		String[] file = getFileAsArray(fname);
		FileOutputStream writer = null;

		try {
			writer = mContext.openFileOutput(fname, Context.MODE_PRIVATE);
		} catch (FileNotFoundException e) {
			// empty
		}

		for (String string : file) {
			if (string == null) {
				break;
			}
			String token = string.substring(0, string.indexOf(sep));
			if (!token0.matches(token)) {
				try {
					writer.write(string.getBytes());
					writer.write(System.getProperty("line.separator")
							.getBytes());
					writer.flush();
				} catch (IOException e) {
					// empty
				}
			}
		}
		try {
			writer.close();
		} catch (IOException e) {
			// empty
		}
	}



	/**
	 *  Creates empty internal file "fname"
	 */
	public int touchFile(String fname) {
		FileOutputStream writer;

		try {
			writer = mContext.openFileOutput(fname, Context.MODE_PRIVATE);
		} catch (FileNotFoundException e) {
			return -1;
		}

		try {
			writer.close();
		} catch (IOException e) {
			return -1;
		}
		return 0;
	}



	/**
	 * write String[] lines to external file fname
	 * @return -1 on error else 0
	 */
	public int writeArrayToExtFile(String[] lines, String fname) {
		FileOutputStream writer;
		File file;

		try {
			file = new File(fname);
            writer = new FileOutputStream(file);
		} catch (FileNotFoundException e) {
			return -1;
		}

		for (String string : lines) {
			try {
				writer.write(string.getBytes());
				writer.write(System.getProperty("line.separator").getBytes());
				writer.flush();
			} catch (IOException e) {
				return -1;
			}
		}

		try {
			writer.close();
		} catch (IOException e) {
			return -1;
		}
		return 0;
	}



	/**
	 *   Writes String[] list to internal file fname
	 **/
	public int writeArrayToFile(String[] list, String fname) {
		List<String> tmp = Arrays.asList(list);
		int val = writeListToFile(tmp, fname);
		return val;
	}



	/**
	 *   Writes List list to internal file fname
	 **/
	public int writeListToFile(List<String> list, String fname) {
		FileOutputStream writer;

		try {
			writer = mContext.openFileOutput(fname, Context.MODE_PRIVATE);
		} catch (FileNotFoundException e) {
			return -1;
		}

		for (String string : list) {
			try {
				writer.write(string.getBytes());
				writer.write(System.getProperty("line.separator").getBytes());
				writer.flush();
			} catch (IOException e) {
				return -1;
			}
		}

		try {
			writer.close();
		} catch (IOException e) {
			return -1;
		}
		return 0;
	}



	/**
	 *   Adds List top to top of internal file fname
	 **/
	public void writeToTopOfFile(List<String> top, String fname){
		List<String> old = getFileAsList(fname);
		old.addAll(0, top);
		writeListToFile(old, fname);
	}


	/**
	 * Inserts/updates line in internal file fname by unique token[0] 
	 * Sep is seperator character in line
	 * Returns -1 on error else 0
	 */
	public int writeUniqLineByToken0(String fname, String line, String sep) {
		String[] file = getFileAsArray(fname);
		FileOutputStream writer = null;

		try {
			writer = mContext.openFileOutput(fname, Context.MODE_PRIVATE);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return -1;
		}

		// get new data tokens
		String tmp[] = line.split(sep);
		String new0 = tmp[0];
		
		try {
			for (String string : file) {
				String tmp2[] = string.split(sep);
				String old0 = tmp2[0];
				if (!old0.matches(new0)) {
					try {
						writer.write(string.getBytes());
						writer.write(System.getProperty("line.separator")
									 .getBytes());
						writer.flush();
					} catch (IOException e) {
						throw new Exception();
					}
				}
			}

			// write new line of data
			try {
				writer.write(line.getBytes());
				writer.write(System.getProperty("line.separator").getBytes());
				writer.flush();
			} catch (IOException e) {
				throw new Exception();
			}
			
			try {
				writer.close();
			} catch (IOException e) {
				throw new Exception();
			}
		} catch (Exception e) {
			writeArrayToFile(file, fname);
			return -1;
		}
			
		return 0;
	}


	/**
	 * Updates line in all matching internal files by unique token[0] 
	 * Filenames match if beginning with prefix
	 * Sep is seperator character in line
	 * Returns -1 on error else 0-=-
	 */
	public int updateInAllByToken0(String line, String prefix, String sep) {
		File internal = mContext.getFilesDir();
		File[] files = internal.listFiles();
		String[] tokens = null;
		FileOutputStream writer = null;

		for (File f : files){
			String fname = f.toString();
			tokens	= fname.split("/");
			fname = tokens[tokens.length - 1];
			String[] file = getFileAsArray(fname);
			try {
				writer = mContext.openFileOutput(fname, Context.MODE_PRIVATE);
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}

			// get new data tokens
			String tmp[] = line.split(sep);
			String new0 = tmp[0];

			for (String string : file) {
				String tmp2[] = string.split(sep);
				String old0 = tmp2[0];
				if (old0.matches(new0)) {
					// write updated line of data
					try {
						writer.write(line.getBytes());
						writer.write(System.getProperty("line.separator").getBytes());
						writer.flush();
					} catch (IOException e) {
						e.printStackTrace();
					}
				} else {
					try {
						writer.write(string.getBytes());
						writer.write(System.getProperty("line.separator").getBytes());
						writer.flush();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
			try {
				writer.close();
			} catch (IOException e) {
				return -1;
			}

		}
		return 0;
	}



	/**
	 * Inserts/updates line by unique token[0] and token[1]
	 * So first two tokens are unique in internal file fname  
	 * Separator of lines in fname is sep
	 * return -1 on error else 0
	 */
	public int writeUniqLineByTokens(String fname, String line, String sep) {
		String[] file = getFileAsArray(fname);
		FileOutputStream writer = null;

		try {
			writer = mContext.openFileOutput(fname, Context.MODE_PRIVATE);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return -1;
		}

		// get new data tokens
		String tmp[] = line.split(sep);
		String new0 = tmp[0];
		String new1 = tmp[1];

		try {
			for (String string : file) {
				String tmp2[] = string.split(sep);
				String old0 = tmp2[0];
				String old1 = tmp2[1];
				if ((!old1.matches(new1)) && (!old0.matches(new0))) {
					try {
						writer.write(string.getBytes());
						writer.write(System.getProperty("line.separator")
									 .getBytes());
						writer.flush();
					} catch (IOException e) {
						throw new Exception();
					}
				}
			}

			// write new line of data
			try {
				writer.write(line.getBytes());
				writer.write(System.getProperty("line.separator").getBytes());
				writer.flush();
			} catch (IOException e) {
				throw new Exception();
			}

			try {
				writer.close();
			} catch (IOException e) {
				throw new Exception();
			} 
		} catch (Exception e) {
			writeArrayToFile(file, fname);
			return -1;
		}
		return 0;
	}


}
